@@ -20,6 +20,8 @@
  *
  *    Static helpers
  *       static l_int32    getNextNonCommentLine()
+ *       static l_int32    getNextNonBlankLine()
+ *       static l_int32    getNextNonDoubleSlashLine()
  *       static l_int32    searchForProtoSignature()
  *       static char      *captureProtoSignature()
  *       static char      *cleanProtoSignature()
@@ -31,24 +33,29 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include "allheaders.h"
 
-static const l_int32 BUF_SIZE = 512;  /* max token size */
+    /* MS VC++ can't handle array initialization with static consts ! */
+#define L_BUF_SIZE      512    /* max token size */
 
 
 static l_int32 getNextNonCommentLine(SARRAY *sa, l_int32 start, l_int32 *pnext);
+static l_int32 getNextNonBlankLine(SARRAY *sa, l_int32 start, l_int32 *pnext);
+static l_int32 getNextNonDoubleSlashLine(SARRAY *sa, l_int32 start,
+            l_int32 *pnext);
 static l_int32 searchForProtoSignature(SARRAY *sa, l_int32 begin,
-	    l_int32 *pstart, l_int32 *pstop, l_int32 *pcharindex,
-	    l_int32 *pfound);
+            l_int32 *pstart, l_int32 *pstop, l_int32 *pcharindex,
+            l_int32 *pfound);
 static char * captureProtoSignature(SARRAY *sa, l_int32 start, l_int32 stop,
-	    l_int32 charindex);
+            l_int32 charindex);
 static char * cleanProtoSignature(char *str);
 static l_int32 skipToEndOfFunction(SARRAY *sa, l_int32 start,
-	    l_int32 charindex, l_int32 *pnext);
+            l_int32 charindex, l_int32 *pnext);
 static l_int32 skipToMatchingBrace(SARRAY *sa, l_int32 start,
-	    l_int32 lbindex, l_int32 *prbline, l_int32 *prbindex);
+            l_int32 lbindex, l_int32 *prbline, l_int32 *prbindex);
 static l_int32 skipToSemicolon(SARRAY *sa, l_int32 start,
-	    l_int32 charindex, l_int32 *pnext);
+            l_int32 charindex, l_int32 *pnext);
 static l_int32 getOffsetForCharacter(SARRAY *sa, l_int32 start, char tchar,
             l_int32 *psoffset, l_int32 *pboffset, l_int32 *ptoffset);
 
@@ -121,36 +128,36 @@ SARRAY    *sa, *saout, *satest;
     PROCNAME("parseForProtos");
 
     if (!filein)
-	return (char *)ERROR_PTR("filein not defined", procName, NULL);
+        return (char *)ERROR_PTR("filein not defined", procName, NULL);
 
         /* Read in the cpp output into memory, one string for each
-	 * line in the file, omitting blank lines.  */
+         * line in the file, omitting blank lines.  */
     strdata = (char *)arrayRead(filein, &nbytes);
     sa = sarrayCreateLinesFromString(strdata, 0);
 
     saout = sarrayCreate(0);
     next = 0;
     while (1) {  /* repeat after each non-static prototype is extracted */
-	searchForProtoSignature(sa, next, &start, &stop, &charindex, &found);
-	if (!found)
-	    break;
-/*	fprintf(stderr, "  start = %d, stop = %d, charindex = %d\n",
-		start, stop, charindex); */
-	str = captureProtoSignature(sa, start, stop, charindex);
+        searchForProtoSignature(sa, next, &start, &stop, &charindex, &found);
+        if (!found)
+            break;
+/*        fprintf(stderr, "  start = %d, stop = %d, charindex = %d\n",
+                start, stop, charindex); */
+        str = captureProtoSignature(sa, start, stop, charindex);
 
-	    /* Make sure it is not static.  Note that 'extern' has
-	     * been prepended to the prototype, so the 'static'
-	     * keyword, if it exists, would be the second word. */
-	satest = sarrayCreateWordsFromString(str);
-	secondword = sarrayGetString(satest, 1, 0);
-	if (strcmp(secondword, "static"))  /* not static */
-	    sarrayAddString(saout, str, 0);
-	else
-	    FREE(str);
-	sarrayDestroy(&satest);
+            /* Make sure it is not static.  Note that 'extern' has
+             * been prepended to the prototype, so the 'static'
+             * keyword, if it exists, would be the second word. */
+        satest = sarrayCreateWordsFromString(str);
+        secondword = sarrayGetString(satest, 1, 0);
+        if (strcmp(secondword, "static"))  /* not static */
+            sarrayAddString(saout, str, 0);
+        else
+            FREE(str);
+        sarrayDestroy(&satest);
 
-	skipToEndOfFunction(sa, stop, charindex, &next);
-	if (next == -1) break;
+        skipToEndOfFunction(sa, stop, charindex, &next);
+        if (next == -1) break;
     }
 
         /* Flatten into a string with newlines between prototypes */
@@ -173,12 +180,13 @@ SARRAY    *sa, *saout, *satest;
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
- *      (1) If there is no next uncommented line, return next = -1
+ *      (1) Skips over all consecutive comment lines, beginning at 'start'
+ *      (2) If all lines to the end are '#' comments, return next = -1
  */
 static l_int32
 getNextNonCommentLine(SARRAY  *sa,
-	              l_int32  start,
-	              l_int32 *pnext)
+                      l_int32  start,
+                      l_int32 *pnext)
 {
 char    *str;
 l_int32  i, n;
@@ -186,20 +194,118 @@ l_int32  i, n;
     PROCNAME("getNextNonCommentLine");
 
     if (!sa)
-	return ERROR_INT("sa not defined", procName, 1);
+        return ERROR_INT("sa not defined", procName, 1);
     if (!pnext)
-	return ERROR_INT("&pnext not defined", procName, 1);
+        return ERROR_INT("&pnext not defined", procName, 1);
+
+        /* Init for situation where this line and all following are comments */
+    *pnext = -1;
 
-    *pnext = -1;  /* init for situation where all succeeding lines are
-		    commented */
     n = sarrayGetCount(sa);
     for (i = start; i < n; i++) {
-	if ((str = sarrayGetString(sa, i, 0)) == NULL)
-	    return ERROR_INT("str not returned; shouldn't happen", procName, 1);
-	if (str[0] != '#') {
-	    *pnext = i;
-	    return 0;
-	}
+        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+            return ERROR_INT("str not returned; shouldn't happen", procName, 1);
+        if (str[0] != '#') {
+            *pnext = i;
+            return 0;
+        }
+    }
+
+    return 0;
+}
+
+
+/* 
+ *  getNextNonBlankLine()
+ *
+ *      Input:  sa (output from cpp, by line)
+ *              start (starting index to search)
+ *              &next (<return> index of first nonblank line after
+ *                     the start line)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Skips over all consecutive blank lines, beginning at 'start'
+ *      (2) A blank line has only whitespace characters (' ', '\t', '\n', '\r')
+ *      (3) If all lines to the end are blank, return next = -1
+ */
+static l_int32
+getNextNonBlankLine(SARRAY  *sa,
+                    l_int32  start,
+                    l_int32 *pnext)
+{
+char    *str;
+l_int32  i, j, n, len;
+
+    PROCNAME("getNextNonBlankLine");
+
+    if (!sa)
+        return ERROR_INT("sa not defined", procName, 1);
+    if (!pnext)
+        return ERROR_INT("&pnext not defined", procName, 1);
+
+        /* Init for situation where this line and all following are blank */
+    *pnext = -1;
+
+    n = sarrayGetCount(sa);
+    for (i = start; i < n; i++) {
+        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+            return ERROR_INT("str not returned; shouldn't happen", procName, 1);
+        len = strlen(str);
+        for (j = 0; j < len; j++) {
+            if (str[j] != ' ' && str[j] != '\t'
+                && str[j] != '\n' && str[j] != '\r') {  /* non-blank */
+                *pnext = i;
+                return 0;
+            }
+        }
+    }
+
+    return 0;
+}
+
+
+/* 
+ *  getNextNonDoubleSlashLine()
+ *
+ *      Input:  sa (output from cpp, by line)
+ *              start (starting index to search)
+ *              &next (<return> index of first uncommented line after
+ *                     the start line)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Skips over all consecutive '//' lines, beginning at 'start'
+ *      (2) If all lines to the end start with '//', return next = -1
+ */
+static l_int32
+getNextNonDoubleSlashLine(SARRAY  *sa,
+                          l_int32  start,
+                          l_int32 *pnext)
+{
+char    *str;
+l_int32  i, n, len;
+
+    PROCNAME("getNextNonDoubleSlashLine");
+
+    if (!sa)
+        return ERROR_INT("sa not defined", procName, 1);
+    if (!pnext)
+        return ERROR_INT("&pnext not defined", procName, 1);
+
+        /* Init for situation where this line and all following
+         * start with '//' */
+    *pnext = -1;
+
+    n = sarrayGetCount(sa);
+    for (i = start; i < n; i++) {
+        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+            return ERROR_INT("str not returned; shouldn't happen", procName, 1);
+        len = strlen(str);
+        if (len < 2 || str[0] != '/' || str[1] != '/') {
+            *pnext = i;
+            return 0;
+        }
     }
 
     return 0;
@@ -221,7 +327,7 @@ l_int32  i, n;
  *      (1) If this returns found == 0, it means that there are no
  *          more function definitions in the file.  Caller must check
  *          this value and exit the loop over the entire cpp file.
- *      (2) This follows plan 3 (see above).  We skip comment
+ *      (2) This follows plan 3 (see above).  We skip comment and blank
  *          lines at the beginning.  Then we don't check for keywords.
  *          Instead, find the relative locations of the first occurrences
  *          of these four tokens: left parenthesis (lp), right
@@ -238,15 +344,13 @@ l_int32  i, n;
  */
 static l_int32
 searchForProtoSignature(SARRAY   *sa,
-	                l_int32   begin,
-	                l_int32  *pstart,
-			l_int32  *pstop,
-			l_int32  *pcharindex,
-			l_int32  *pfound)
+                        l_int32   begin,
+                        l_int32  *pstart,
+                        l_int32  *pstop,
+                        l_int32  *pcharindex,
+                        l_int32  *pfound)
 {
-char    *str, *firstword, *strtemp;
-SARRAY   *satok, *satemp;
-l_int32  i, n, next, rbline, rbindex, scline;
+l_int32  next, rbline, rbindex, scline;
 l_int32  soffsetlp, soffsetrp, soffsetlb, soffsetsc;
 l_int32  boffsetlp, boffsetrp, boffsetlb, boffsetsc;
 l_int32  toffsetlp, toffsetrp, toffsetlb, toffsetsc;
@@ -254,60 +358,83 @@ l_int32  toffsetlp, toffsetrp, toffsetlb, toffsetsc;
     PROCNAME("searchForProtoSignature");
 
     if (!sa)
-	return ERROR_INT("sa not defined", procName, 1);
+        return ERROR_INT("sa not defined", procName, 1);
     if (!pstart)
-	return ERROR_INT("&start not defined", procName, 1);
+        return ERROR_INT("&start not defined", procName, 1);
     if (!pstop)
-	return ERROR_INT("&stop not defined", procName, 1);
+        return ERROR_INT("&stop not defined", procName, 1);
     if (!pcharindex)
-	return ERROR_INT("&charindex not defined", procName, 1);
+        return ERROR_INT("&charindex not defined", procName, 1);
     if (!pfound)
-	return ERROR_INT("&found not defined", procName, 1);
+        return ERROR_INT("&found not defined", procName, 1);
 
     *pfound = FALSE;
 
     while (1) {
+
+            /* Skip over sequential '#' comment lines */
         getNextNonCommentLine(sa, begin, &next);
-	if (next == -1) break;
+        if (next == -1) return 0;
+        if (next != begin) {
+            begin = next;
+            continue;
+        }
 
-	    /* Search for specific character sequence patterns */
-	getOffsetForCharacter(sa, next, '(', &soffsetlp, &boffsetlp,
-		&toffsetlp);
-	getOffsetForCharacter(sa, next, ')', &soffsetrp, &boffsetrp,
-		&toffsetrp);
-	getOffsetForCharacter(sa, next, '{', &soffsetlb, &boffsetlb,
-		&toffsetlb);
-	getOffsetForCharacter(sa, next, ';', &soffsetsc, &boffsetsc,
-		&toffsetsc);
+            /* Skip over sequential blank lines */
+        getNextNonBlankLine(sa, begin, &next);
+        if (next == -1) return 0;
+        if (next != begin) {
+            begin = next;
+            continue;
+        }
 
-	    /* First weed out cases where lp, rp and lb are not all found */
-	if (soffsetlp == -1 || soffsetrp == -1 || soffsetlb == -1)
-	    break;
+            /* Skip over sequential lines starting with '//' */
+        getNextNonDoubleSlashLine(sa, begin, &next);
+        if (next == -1) return 0;
+        if (next != begin) {
+            begin = next;
+            continue;
+        }
 
-	    /* Check if a left brace occurs before a left parenthesis;
-	     * if so, skip it */
-	if (toffsetlb < toffsetlp) {  
+            /* Search for specific character sequence patterns */
+        getOffsetForCharacter(sa, next, '(', &soffsetlp, &boffsetlp,
+                &toffsetlp);
+        getOffsetForCharacter(sa, next, ')', &soffsetrp, &boffsetrp,
+                &toffsetrp);
+        getOffsetForCharacter(sa, next, '{', &soffsetlb, &boffsetlb,
+                &toffsetlb);
+        getOffsetForCharacter(sa, next, ';', &soffsetsc, &boffsetsc,
+                &toffsetsc);
+
+            /* First weed out cases where lp, rp and lb are not all found */
+        if (soffsetlp == -1 || soffsetrp == -1 || soffsetlb == -1)
+            break;
+
+            /* Check if a left brace occurs before a left parenthesis;
+             * if so, skip it */
+        if (toffsetlb < toffsetlp) {  
             skipToMatchingBrace(sa, next + soffsetlb, boffsetlb,
-		&rbline, &rbindex);
+                &rbline, &rbindex);
             skipToSemicolon(sa, rbline, rbindex, &scline);
-	    begin = scline + 1;
-	    continue;
-	}
+            begin = scline + 1;
+            continue;
+        }
 
-	    /* Check if a semicolon occurs before a left brace;
-	     * if so, skip it */
-	if ((soffsetsc != -1) && toffsetsc < toffsetlb) {  
+            /* Check if a semicolon occurs before a left brace or
+             * a left parenthesis; if so, skip it */
+        if ((soffsetsc != -1)
+            && (toffsetsc < toffsetlb) || (toffsetsc < toffsetlp)) {  
             skipToSemicolon(sa, next, 0, &scline);
-	    begin = scline + 1;
-	    continue;
-	}
+            begin = scline + 1;
+            continue;
+        }
 
-	    /* OK, it should be a function definition */
-	*pstart = next;
-	*pstop = next + soffsetrp;
-	*pcharindex = boffsetrp;
-	*pfound = TRUE;
-	break;
+            /* OK, it should be a function definition */
+        *pstart = next;
+        *pstop = next + soffsetrp;
+        *pcharindex = boffsetrp;
+        *pfound = TRUE;
+        break;
     }
 
     return 0;
@@ -333,18 +460,18 @@ captureProtoSignature(SARRAY  *sa,
                       l_int32  charindex)
 {
 char    *str, *newstr, *protostr, *cleanstr;
-SARRAY  *sap, *satemp;
+SARRAY  *sap;
 l_int32  i;
 
     PROCNAME("captureProtoSignature");
 
     if (!sa)
-	return (char *)ERROR_PTR("sa not defined", procName, NULL);
+        return (char *)ERROR_PTR("sa not defined", procName, NULL);
 
     sap = sarrayCreate(0);
     for (i = start; i < stop; i++) {
         str = sarrayGetString(sa, i, 1);
-	sarrayAddString(sap, str, 0);
+        sarrayAddString(sap, str, 0);
     }
     str = sarrayGetString(sa, stop, 1);
     str[charindex + 1] = '\0';
@@ -374,14 +501,14 @@ static char *
 cleanProtoSignature(char *instr)
 {
 char    *str, *cleanstr;
-char     buf[BUF_SIZE];
+char     buf[L_BUF_SIZE];
 l_int32  i, j, nwords, nchars, index, len;
 SARRAY  *sa, *saout;
 
     PROCNAME("cleanProtoSignature");
 
     if (!instr)
-	return (char *)ERROR_PTR("instr not defined", procName, NULL);
+        return (char *)ERROR_PTR("instr not defined", procName, NULL);
 
     sa = sarrayCreateWordsFromString(instr);
     nwords = sarrayGetCount(sa);
@@ -389,29 +516,29 @@ SARRAY  *sa, *saout;
     sarrayAddString(saout, "extern", 1);
     for (i = 0; i < nwords; i++) {
         str = sarrayGetString(sa, i, 0);
-	nchars = strlen(str);
-	index = 0;
-	for (j = 0; j < nchars; j++) {
-	    if (index > BUF_SIZE - 6)
-		return (char *)ERROR_PTR("token too large", procName, NULL);
-	    if (str[j] == '(') {
-		buf[index++] = ' ';
-		buf[index++] = '(';
-		buf[index++] = ' ';
-	    }
-	    else if (str[j] == ')') {
-		buf[index++] = ' ';
-		buf[index++] = ')';
-	    }
-	    else 
-		buf[index++] = str[j];
-	}
-	buf[index] = '\0';
-	sarrayAddString(saout, buf, 1);
+        nchars = strlen(str);
+        index = 0;
+        for (j = 0; j < nchars; j++) {
+            if (index > L_BUF_SIZE - 6)
+                return (char *)ERROR_PTR("token too large", procName, NULL);
+            if (str[j] == '(') {
+                buf[index++] = ' ';
+                buf[index++] = '(';
+                buf[index++] = ' ';
+            }
+            else if (str[j] == ')') {
+                buf[index++] = ' ';
+                buf[index++] = ')';
+            }
+            else 
+                buf[index++] = str[j];
+        }
+        buf[index] = '\0';
+        sarrayAddString(saout, buf, 1);
     }
 
         /* Flatten to a prototype string with spaces added after
-	 * each word, and remove the last space */
+         * each word, and remove the last space */
     cleanstr = sarrayToString(saout, 2);
     len = strlen(cleanstr);
     cleanstr[len - 1] = '\0';
@@ -443,16 +570,16 @@ l_int32 soffsetlb, boffsetlb, toffsetlb;
     PROCNAME("skipToEndOfFunction");
 
     if (!sa)
-	return ERROR_INT("sa not defined", procName, 1);
+        return ERROR_INT("sa not defined", procName, 1);
     if (!pnext)
-	return ERROR_INT("&next not defined", procName, 1);
+        return ERROR_INT("&next not defined", procName, 1);
 
     getOffsetForCharacter(sa, start, '{', &soffsetlb, &boffsetlb,
-		&toffsetlb);
+                &toffsetlb);
     skipToMatchingBrace(sa, start + soffsetlb, boffsetlb, &end, &rbindex);
     if (end == -1) {  /* shouldn't happen! */
-	*pnext = -1;
-	return 1;
+        *pnext = -1;
+        return 1;
     }
 
     *pnext = end + 1;
@@ -487,11 +614,11 @@ l_int32  i, j, jstart, n, sumbrace, found, instring, nchars;
     PROCNAME("skipToMatchingBrace");
 
     if (!sa)
-	return ERROR_INT("sa not defined", procName, 1);
+        return ERROR_INT("sa not defined", procName, 1);
     if (!pstop)
-	return ERROR_INT("&stop not defined", procName, 1);
+        return ERROR_INT("&stop not defined", procName, 1);
     if (!prbindex)
-	return ERROR_INT("&rbindex not defined", procName, 1);
+        return ERROR_INT("&rbindex not defined", procName, 1);
 
     instring = 0;  /* init to FALSE; toggle on double quotes */
     *pstop = -1;
@@ -499,35 +626,35 @@ l_int32  i, j, jstart, n, sumbrace, found, instring, nchars;
     sumbrace = 1;
     found = FALSE;
     for (i = start; i < n; i++) {
-	str = sarrayGetString(sa, i, 0);
-	jstart = 0;
-	if (i == start)
-	    jstart = lbindex + 1;
-	nchars = strlen(str);
-	for (j = jstart; j < nchars; j++) {
-		/* Toggle the instring state every time you encounter
-		 * a double quote that is NOT escaped. */
-	    if (j == jstart && str[j] == '\"')
-		instring = 1 - instring;
-	    if (j > jstart && str[j] == '\"' && str[j-1] != '\\')
-		instring = 1 - instring;
-	        /* Record the braces if they are neither a literal character
-		 * nor within a string. */
-	    if (str[j] == '{' && str[j+1] != '\'' && !instring)
-		sumbrace++;
-	    else if (str[j] == '}' && str[j+1] != '\'' && !instring) {
-		sumbrace--;
-		if (sumbrace == 0) {
-		    found = TRUE;
-		    *prbindex = j;
-		    break;
-		}
-	    }
-	}
-	if (found) {
-	    *pstop = i;
-	    return 0;
-	}
+        str = sarrayGetString(sa, i, 0);
+        jstart = 0;
+        if (i == start)
+            jstart = lbindex + 1;
+        nchars = strlen(str);
+        for (j = jstart; j < nchars; j++) {
+                /* Toggle the instring state every time you encounter
+                 * a double quote that is NOT escaped. */
+            if (j == jstart && str[j] == '\"')
+                instring = 1 - instring;
+            if (j > jstart && str[j] == '\"' && str[j-1] != '\\')
+                instring = 1 - instring;
+                /* Record the braces if they are neither a literal character
+                 * nor within a string. */
+            if (str[j] == '{' && str[j+1] != '\'' && !instring)
+                sumbrace++;
+            else if (str[j] == '}' && str[j+1] != '\'' && !instring) {
+                sumbrace--;
+                if (sumbrace == 0) {
+                    found = TRUE;
+                    *prbindex = j;
+                    break;
+                }
+            }
+        }
+        if (found) {
+            *pstop = i;
+            return 0;
+        }
     }
 
     return ERROR_INT("matching right brace not found", procName, 1);
@@ -540,7 +667,7 @@ l_int32  i, j, jstart, n, sumbrace, found, instring, nchars;
  *      Input:  sa (output from cpp, by line)
  *              start (index of starting line to search)
  *              charindex (starting char index for search)
- *              &next (index of line following the ending ';'
+ *              &next (index of line containing the next ';')
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -561,29 +688,29 @@ l_int32  i, j, n, jstart, nchars, found;
     PROCNAME("skipToSemicolon");
 
     if (!sa)
-	return ERROR_INT("sa not defined", procName, 1);
+        return ERROR_INT("sa not defined", procName, 1);
     if (!pnext)
-	return ERROR_INT("&next not defined", procName, 1);
+        return ERROR_INT("&next not defined", procName, 1);
 
     *pnext = -1;
     n = sarrayGetCount(sa);
     found = FALSE;
     for (i = start; i < n; i++) {
-	str = sarrayGetString(sa, i, 0);
-	jstart = 0;
-	if (i == start)
-	    jstart = charindex + 1;
-	nchars = strlen(str);
-	for (j = jstart; j < nchars; j++) {
-	    if (str[j] == ';') {
-		found = TRUE;;
-	        break;
-	    }
-	}
-	if (found) {
-	    *pnext = i + 1;
-	    return 0;
-	}
+        str = sarrayGetString(sa, i, 0);
+        jstart = 0;
+        if (i == start)
+            jstart = charindex + 1;
+        nchars = strlen(str);
+        for (j = jstart; j < nchars; j++) {
+            if (str[j] == ';') {
+                found = TRUE;;
+                break;
+            }
+        }
+        if (found) {
+            *pnext = i;
+            return 0;
+        }
     }
 
     return ERROR_INT("semicolon not found", procName, 1);
@@ -608,8 +735,8 @@ l_int32  i, j, n, jstart, nchars, found;
  *      (1) We are searching for the first instance of 'tchar', starting
  *          at the beginning of the string indexed by start.
  *      (2) If the character is not found, soffset is returned as -1,
- *          and the other offsets are not set, so the caller must
- *          check the value of soffset.
+ *          and the other offsets are set to very large numbers.  The
+ *          caller must check the value of soffset.
  *      (3) This is only used in contexts where it is not necessary to
  *          consider if the character is inside a string.
  */
@@ -627,36 +754,38 @@ l_int32  i, j, n, nchars, totchars, found;
     PROCNAME("getOffsetForCharacter");
 
     if (!sa)
-	return ERROR_INT("sa not defined", procName, 1);
+        return ERROR_INT("sa not defined", procName, 1);
     if (!psoffset)
-	return ERROR_INT("&soffset not defined", procName, 1);
+        return ERROR_INT("&soffset not defined", procName, 1);
     if (!pboffset)
-	return ERROR_INT("&boffset not defined", procName, 1);
+        return ERROR_INT("&boffset not defined", procName, 1);
     if (!ptoffset)
-	return ERROR_INT("&toffset not defined", procName, 1);
+        return ERROR_INT("&toffset not defined", procName, 1);
 
     *psoffset = -1;  /* init to not found */
+    *pboffset = 100000000;
+    *ptoffset = 100000000;
 
     n = sarrayGetCount(sa);
     found = FALSE; 
     totchars = 0;
     for (i = start; i < n; i++) {
         if ((str = sarrayGetString(sa, i, 0)) == NULL)
-	    return ERROR_INT("str not returned; shouldn't happen", procName, 1);
-	nchars = strlen(str);
-	for (j = 0; j < nchars; j++) {
-	    if (str[j] == tchar) {
-		found = TRUE;
-		break;
-	    }
-	}
-	if (found)
-	    break;
-	totchars += nchars;
+            return ERROR_INT("str not returned; shouldn't happen", procName, 1);
+        nchars = strlen(str);
+        for (j = 0; j < nchars; j++) {
+            if (str[j] == tchar) {
+                found = TRUE;
+                break;
+            }
+        }
+        if (found)
+            break;
+        totchars += nchars;
     }
 
     if (found) {
-	*psoffset = i - start;
+        *psoffset = i - start;
         *pboffset = j;
         *ptoffset = totchars + j;
     }
