@@ -34,6 +34,7 @@
  *           char      *stringNew()
  *           l_int32    stringReplace()
  *           char      *stringJoin()
+ *           char      *strtokSafe()
  *
  *       find and replace procs
  *           char      *stringRemoveChars()
@@ -77,18 +78,18 @@
 #include "allheaders.h"
 
 
-/*------------------------------------------------------------------*
- *               Error, warning and info message procs              *
- *                                                                  *
- *          ---------------------  N.B. ---------------------       *
- *                                                                  *
- *  (1) These functions all print messages to stderr.               *
- *                                                                  *
- *  (2) They must be invoked only by macros, which are in           *
- *      environ.h, so that the print output can be disabled         *
- *      at compile time, using -DNO_CONSOLE_IO.                     *
- *                                                                  *
- *------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*
+ *                 Error, warning and info message procs                *
+ *                                                                      *
+ *            ---------------------  N.B. ---------------------         *
+ *                                                                      *
+ *    (1) These functions all print messages to stderr.                 *
+ *                                                                      *
+ *    (2) They must be invoked only by macros, which are in             *
+ *        environ.h, so that the print output can be disabled           *
+ *        at compile time, using -DNO_CONSOLE_IO.                       *
+ *                                                                      *
+ *----------------------------------------------------------------------*/
 /*!
  *  l_errorInt()
  *
@@ -100,7 +101,7 @@
 l_int32
 l_errorInt(const char  *msg, 
            const char  *procname, 
-	   l_int32      ival)
+           l_int32      ival)
 {
     fprintf(stderr, "Error in %s: %s\n", procname, msg);
     return ival;
@@ -117,8 +118,8 @@ l_errorInt(const char  *msg,
  */
 l_float32
 l_errorFloat(const char  *msg, 
-	     const char  *procname, 
-	     l_float32    fval)
+             const char  *procname, 
+             l_float32    fval)
 {
     fprintf(stderr, "Error in %s: %s\n", procname, msg);
     return fval;
@@ -136,10 +137,10 @@ l_errorFloat(const char  *msg,
 void *
 l_errorPtr(const char  *msg,
            const char  *procname, 
-	   void        *pval)
+           void        *pval)
 {
     fprintf(stderr, "Error in %s: %s\n", procname, msg);
-    return (void*)pval;
+    return pval;
 }
 
 
@@ -183,26 +184,26 @@ l_warning(const char  *msg,
 void
 l_warningInt(const char  *msg,
              const char  *procname, 
-	     l_int32      ival)
+             l_int32      ival)
 {
 l_int32  bufsize;
 char    *charbuf;
 
     if (!msg || !procname) {
-	ERROR_VOID("msg or procname not defined in l_warningInt()", procname);
-	return;
+        ERROR_VOID("msg or procname not defined in l_warningInt()", procname);
+        return;
     }
 
     bufsize = strlen(msg) + strlen(procname) + 128;
     if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
-	ERROR_VOID("charbuf not made in l_warningInt()", procname);
-	return;
+        ERROR_VOID("charbuf not made in l_warningInt()", procname);
+        return;
     }
 
     sprintf(charbuf, "Warning in %s: %s\n", procname, msg);
     fprintf(stderr, charbuf, ival);
 
-    FREE((void *)charbuf);
+    FREE(charbuf);
     return;
 }
 
@@ -238,20 +239,20 @@ l_int32  bufsize;
 char    *charbuf;
 
     if (!msg || !procname) {
-	ERROR_VOID("msg or procname not defined in l_infoInt()", procname);
-	return;
+        ERROR_VOID("msg or procname not defined in l_infoInt()", procname);
+        return;
     }
 
     bufsize = strlen(msg) + strlen(procname) + 128;
     if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
-	ERROR_VOID("charbuf not made in l_infoInt()", procname);
-	return;
+        ERROR_VOID("charbuf not made in l_infoInt()", procname);
+        return;
     }
 
     sprintf(charbuf, "Info in %s: %s\n", procname, msg);
     fprintf(stderr, charbuf, ival);
 
-    FREE((void *)charbuf);
+    FREE(charbuf);
     return;
 }
 
@@ -273,20 +274,20 @@ l_int32  bufsize;
 char    *charbuf;
 
     if (!msg || !procname) {
-	ERROR_VOID("msg or procname not defined in l_infoInt2()", procname);
-	return;
+        ERROR_VOID("msg or procname not defined in l_infoInt2()", procname);
+        return;
     }
 
     bufsize = strlen(msg) + strlen(procname) + 128;
     if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
-	ERROR_VOID("charbuf not made in l_infoInt2()", procname);
-	return;
+        ERROR_VOID("charbuf not made in l_infoInt2()", procname);
+        return;
     }
 
     sprintf(charbuf, "Info in %s: %s\n", procname, msg);
     fprintf(stderr, charbuf, ival1, ival2);
 
-    FREE((void *)charbuf);
+    FREE(charbuf);
     return;
 }
 
@@ -307,20 +308,20 @@ l_int32  bufsize;
 char    *charbuf;
 
     if (!msg || !procname) {
-	ERROR_VOID("msg or procname not defined in l_infoFloat()", procname);
-	return;
+        ERROR_VOID("msg or procname not defined in l_infoFloat()", procname);
+        return;
     }
 
     bufsize = strlen(msg) + strlen(procname) + 128;
     if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
-	ERROR_VOID("charbuf not made in l_infoFloat()", procname);
-	return;
+        ERROR_VOID("charbuf not made in l_infoFloat()", procname);
+        return;
     }
 
     sprintf(charbuf, "from %s: %s\n", procname, msg);
     fprintf(stderr, charbuf, fval);
 
-    FREE((void *)charbuf);
+    FREE(charbuf);
     return;
 }
 
@@ -342,20 +343,20 @@ l_int32  bufsize;
 char    *charbuf;
 
     if (!msg || !procname) {
-	ERROR_VOID("msg or procname not defined in l_infoFloat2()", procname);
-	return;
+        ERROR_VOID("msg or procname not defined in l_infoFloat2()", procname);
+        return;
     }
 
     bufsize = strlen(msg) + strlen(procname) + 128;
     if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
-	ERROR_VOID("charbuf not made in l_infoFloat()", procname);
-	return;
+        ERROR_VOID("charbuf not made in l_infoFloat()", procname);
+        return;
     }
 
     sprintf(charbuf, "from %s: %s\n", procname, msg);
     fprintf(stderr, charbuf, fval1, fval2);
 
-    FREE((void *)charbuf);
+    FREE(charbuf);
     return;
 }
 
@@ -378,10 +379,10 @@ char  *dest;
     PROCNAME("stringNew");
 
     if (!src)
-	return (char *)ERROR_PTR("src not defined", procName, NULL);
+        return (char *)ERROR_PTR("src not defined", procName, NULL);
     
     if ((dest = (char *)CALLOC(strlen(src) + 2, sizeof(char))) == NULL)
-	return (char *)ERROR_PTR("dest not made", procName, NULL);
+        return (char *)ERROR_PTR("dest not made", procName, NULL);
     strcpy(dest, src);
 
     return dest;
@@ -395,15 +396,14 @@ char  *dest;
  *              src string
  *      Return: 0 if OK; 1 on error
  *
- *  Action: (1) frees any existing dest string
- *          (2) puts a copy of src string in the dest
- *
- *  Nonstandard usage: does reasonable thing if either or both
- *                     strings are null
+ *  Notes:
+ *      (1) Frees any existing dest string
+ *      (2) Puts a copy of src string in the dest
+ *      (3) If either or both strings are null, does the reasonable thing.
  */
 l_int32
 stringReplace(char       **pdest,
-	      const char  *src)
+              const char  *src)
 {
 char  *scopy;
 
@@ -413,16 +413,16 @@ char  *scopy;
         return ERROR_INT("pdest not defined", procName, 1);
 
     if (*pdest)
-	FREE((void *)(*pdest));
+        FREE(*pdest);
     
     if (src) {
-	if ((scopy = (char *)CALLOC(strlen(src) + 2, sizeof(char))) == NULL)
-	    return ERROR_INT("scopy not made", procName, 1);
-	strcpy(scopy, src);
-	*pdest = scopy;
+        if ((scopy = (char *)CALLOC(strlen(src) + 2, sizeof(char))) == NULL)
+            return ERROR_INT("scopy not made", procName, 1);
+        strcpy(scopy, src);
+        *pdest = scopy;
     }
     else
-	*pdest = NULL;
+        *pdest = NULL;
 
     return 0;
 }
@@ -433,17 +433,16 @@ char  *scopy;
  *
  *      Input:  src1 string (<optional>)
  *              src2 string (<optional>)
- *
  *      Return: concatenated string, or null on error
  *
- *  Safe version of strcat.
- *  Note: It is not an error if either or both of the strings
- *        are empty, or if either or both the pointers are null.
- *
+ *  Notes:
+ *      (1) This is the safe version of strcat; it makes a new string.
+ *      (2) It is not an error if either or both of the strings
+ *          are empty, or if either or both the pointers are null.
  */
 char *
 stringJoin(const char  *src1, 
-	   const char  *src2)
+           const char  *src2)
 {
 char    *dest;
 l_int32  srclen1, srclen2, destlen;
@@ -452,22 +451,118 @@ l_int32  srclen1, srclen2, destlen;
 
     srclen1 = srclen2 = 0;
     if (src1)
-	srclen1 = strlen(src1);
+        srclen1 = strlen(src1);
     if (src2)
-	srclen2 = strlen(src2);
+        srclen2 = strlen(src2);
     destlen = srclen1 + srclen2 + 3;
 
     if ((dest = (char *)CALLOC(destlen, sizeof(char))) == NULL)
         return (char *)ERROR_PTR("calloc fail for dest", procName, NULL);
 
     if (src1)
-	strcpy(dest, src1);
+        strcpy(dest, src1);
     if (src2)
-	strcat(dest, src2);
+        strcat(dest, src2);
     return dest;
 }
 
 
+/*!
+ *  strtokSafe()
+ *
+ *      Input:  cstr (input string to be parsed;
+ *                    use NULL after the first call)
+ *              seps (a string of character separators)
+ *              &saveptr (<return> ptr to the next char after
+ *                        the last encountered separator)
+ *      Return: substr (a new string from the previous saveptr to the first
+ *                      separator character), or NULL if end of cstr.
+ *
+ *  Notes:
+ *      (1) This is a thread-safe implementation of strtok.
+ *      (2) It has the same interface as strtok_r.
+ *      (3) It differs from strtok_r in usage in two respects:
+ *          (a) the input string is not altered
+ *          (b) each returned substring is newly allocated and must
+ *              be freed after use.
+ *      (4) Let me repeat that.  This is "safe" because the input
+ *          string is not altered and because each returned string
+ *          is newly allocated on the heap.
+ *      (5) It is here because, surprisingly, some C libraries don't
+ *          include strtok_r.
+ *      (6) Important usage points:
+ *          - Input the string to be parsed on the first invocation.
+ *          - Then input NULL after that; the value returned in saveptr
+ *            is used in all subsequent calls.
+ *      (7) This is only slightly slower than strtok_k.
+ */
+char *
+strtokSafe(char        *cstr,
+           const char  *seps,
+           char       **psaveptr)
+{
+char     nextc;
+char    *start, *substr;
+l_int32  istart, i, j, nchars;
+
+    PROCNAME("strtokSafe");
+
+    if (!seps)
+        return (char *)ERROR_PTR("seps not defined", procName, NULL);
+    if (!psaveptr)
+        return (char *)ERROR_PTR("&saveptr not defined", procName, NULL);
+
+    if (!cstr)
+        start = *psaveptr;
+    else
+        start = cstr;
+    if (!start)  /* nothing to do */
+        return NULL;
+
+        /* First time, scan for the first non-sep character */
+    istart = 0;
+    if (cstr) {
+        for (istart = 0;; istart++) {
+            if ((nextc = start[istart]) == '\0') {
+                *psaveptr = NULL;  /* in case caller doesn't check ret value */
+                return NULL;
+            }
+            if (!strchr(seps, nextc))
+                break;
+        }
+    }
+
+        /* Scan through, looking for a sep character; if none is
+         * found, 'i' will be at the end of the string. */
+    for (i = istart;; i++) {
+        if ((nextc = start[i]) == '\0')
+            break;
+        if (strchr(seps, nextc))
+            break;
+    }
+
+        /* Save the substring */
+    nchars = i - istart;
+    substr = (char *)CALLOC(nchars + 1, sizeof(char));
+    strncpy(substr, start + istart, nchars);
+
+        /* Look for the next non-sep character.
+         * If this is the last substring, return a null saveptr. */
+    for (j = i;; j++) {
+        if ((nextc = start[j]) == '\0') {
+            *psaveptr = NULL;  /* no more non-sep characters */
+            break;
+        }
+        if (!strchr(seps, nextc)) {
+            *psaveptr = start + j;  /* start here on next call */
+                break;
+        }
+    }
+
+    return substr;
+}
+
+
 /*--------------------------------------------------------------------*
  *                       Find and replace procs                       *
  *--------------------------------------------------------------------*/
@@ -489,12 +584,12 @@ l_int32  nsrc, i, k;
     PROCNAME("stringRemoveChars");
 
     if (!src)
-	return (char *)ERROR_PTR("src not defined", procName, NULL);
+        return (char *)ERROR_PTR("src not defined", procName, NULL);
     if (!remchars)
         return stringNew(src);
     
     if ((dest = (char *)CALLOC(strlen(src) + 1, sizeof(char))) == NULL)
-	return (char *)ERROR_PTR("dest not made", procName, NULL);
+        return (char *)ERROR_PTR("dest not made", procName, NULL);
     nsrc = strlen(src);
     for (i = 0, k = 0; i < nsrc; i++) {
         ch = src[i];
@@ -542,11 +637,11 @@ l_int32  nsrc, nsub1, nsub2, len, npre, loc;
     PROCNAME("stringReplaceSubstr");
 
     if (!src)
-	return (char *)ERROR_PTR("src not defined", procName, NULL);
+        return (char *)ERROR_PTR("src not defined", procName, NULL);
     if (!sub1)
-	return (char *)ERROR_PTR("sub1 not defined", procName, NULL);
+        return (char *)ERROR_PTR("sub1 not defined", procName, NULL);
     if (!sub2)
-	return (char *)ERROR_PTR("sub2 not defined", procName, NULL);
+        return (char *)ERROR_PTR("sub2 not defined", procName, NULL);
     
     if (pfound)
         *pfound = 0;
@@ -565,7 +660,7 @@ l_int32  nsrc, nsub1, nsub2, len, npre, loc;
     nsub2 = strlen(sub2);
     len = nsrc + nsub2 - nsub1;
     if ((dest = (char *)CALLOC(len + 1, sizeof(char))) == NULL)
-	return (char *)ERROR_PTR("dest not made", procName, NULL);
+        return (char *)ERROR_PTR("dest not made", procName, NULL);
     npre = ptr - src;
     memcpy(dest, src, npre);
     strcpy(dest + npre, sub2);
@@ -605,11 +700,11 @@ l_int32  loc;
     PROCNAME("stringReplaceEachSubstr");
 
     if (!src)
-	return (char *)ERROR_PTR("src not defined", procName, NULL);
+        return (char *)ERROR_PTR("src not defined", procName, NULL);
     if (!sub1)
-	return (char *)ERROR_PTR("sub1 not defined", procName, NULL);
+        return (char *)ERROR_PTR("sub1 not defined", procName, NULL);
     if (!sub2)
-	return (char *)ERROR_PTR("sub2 not defined", procName, NULL);
+        return (char *)ERROR_PTR("sub2 not defined", procName, NULL);
 
     if (pcount)
         *pcount = 0;
@@ -624,7 +719,7 @@ l_int32  loc;
         newstr = stringReplaceSubstr(currstr, sub1, sub2, NULL, &loc);
         if (!newstr)
             return currstr;
-        FREE((void *)currstr);
+        FREE(currstr);
         if (pcount)
             (*pcount)++;
     }
@@ -643,7 +738,7 @@ l_int32  loc;
  *              &found (<return> 1 if sequence is found; 0 otherwise)
  *      Return: 0 if OK, 1 on error
  *
- *  Note:
+ *  Notes:
  *      (1) The byte arrays 'data' and 'sequence' are not C strings,
  *          as they can contain null bytes.  Therefore, for each
  *          we must give the length of the array.
@@ -700,26 +795,31 @@ l_int32  i, j, found, lastpos;
 /*!
  *  reallocNew()
  *
- *      Input:  &indata (<used and returned>)
+ *      Input:  &indata (<optional>; nulls indata)
  *              size of input data to be copied (bytes)
  *              size of data to be reallocated (bytes)
  *      Return: ptr to new data, or null on error
  *
  *  Action: !N.B. (3) and (4)!
- *      (1) allocates memory, initialized to 0
- *      (2) copies as much of the input data as possible
+ *      (1) Allocates memory, initialized to 0
+ *      (2) Copies as much of the input data as possible
  *          to the new block, truncating the copy if necessary
- *      (3) frees the input data
- *      (4) zeroes the input data ptr
+ *      (3) Frees the input data
+ *      (4) Zeroes the input data ptr
  *
- *  Notes on nonstandard use:
- *      (1) if newsize <=0, just frees input data and nulls ptr
- *      (2) if input ptr is null, just callocs new memory
+ *  Notes:
+ *      (1) If newsize <=0, just frees input data and nulls ptr
+ *      (2) If input ptr is null, just callocs new memory
+ *      (3) This differs from realloc in that it always allocates
+ *          new memory (if newsize > 0) and initializes it to 0,
+ *          it requires the amount of old data to be copied,
+ *          and it takes the address of the input ptr and
+ *          nulls the handle.
  */
 void *
 reallocNew(void   **pindata,
-	   l_int32  oldsize,
-	   l_int32  newsize)
+           l_int32  oldsize,
+           l_int32  newsize)
 {
 l_int32  minsize;
 void    *indata; 
@@ -728,33 +828,31 @@ void    *newdata;
     PROCNAME("reallocNew");
 
     if (!pindata)
-	return ERROR_PTR("input data not defined", procName, NULL);
-
+        return ERROR_PTR("input data not defined", procName, NULL);
     indata = *pindata;
 
     if (newsize <= 0) {   /* nonstandard usage */
-	if (indata) {
-	    FREE((void *)indata);
-	    *pindata = NULL;
-	}
-	return NULL;
+        if (indata) {
+            FREE(indata);
+            *pindata = NULL;
+        }
+        return NULL;
     }
 
     if (!indata)   /* nonstandard usage */
     {
-	if ((newdata = (void *)CALLOC(1, newsize)) == NULL)
-	    return ERROR_PTR("newdata not made", procName, NULL);
-	return newdata;
+        if ((newdata = (void *)CALLOC(1, newsize)) == NULL)
+            return ERROR_PTR("newdata not made", procName, NULL);
+        return newdata;
     }
 
-	/* standard usage */
+        /* Standard usage */
     if ((newdata = (void *)CALLOC(1, newsize)) == NULL)
-	return ERROR_PTR("newdata not made", procName, NULL);
-
+        return ERROR_PTR("newdata not made", procName, NULL);
     minsize = L_MIN(oldsize, newsize);
     memcpy((char *)newdata, (char *)indata, minsize);
 
-    FREE((void *)indata);
+    FREE(indata);
     *pindata = NULL;
 
     return newdata;
@@ -774,7 +872,7 @@ void    *newdata;
  */
 l_uint8 *
 arrayRead(const char  *fname, 
- 	  l_int32     *pnbytes)
+           l_int32     *pnbytes)
 {
 l_uint8  *data;
 FILE     *fp;
@@ -805,12 +903,13 @@ FILE     *fp;
  *      Return: null-terminated array, or null on error
  *              (reading 0 bytes is not an error)
  *
- *  N.B. -- Side effect:
- *      This always re-positions stream ptr to the beginning of the file.
+ *  Notes:
+ *      (1) N.B.: as a side effect, this always re-positions the
+ *          stream ptr to the beginning of the file.
  */
 l_uint8 *
 arrayReadStream(FILE     *fp, 
-	        l_int32  *pnbytes)
+                l_int32  *pnbytes)
 {
 l_uint8  *data;
 
@@ -845,7 +944,7 @@ l_int32  nbytes, pos;
     PROCNAME("nbytesInFile");
 
     if (!fp)
-	return ERROR_INT("stream not open", procName, 0);
+        return ERROR_INT("stream not open", procName, 0);
 
     pos = ftell(fp);          /* initial position */
     fseek(fp, 0, SEEK_END);   /* EOF */
@@ -871,28 +970,28 @@ l_int32  nbytes, pos;
  */
 l_int32
 arrayWrite(const char  *filename,
-	   const char  *operation,
-	   void        *data,
-	   l_int32      nbytes)
+           const char  *operation,
+           void        *data,
+           l_int32      nbytes)
 {
 FILE  *fp;
 
     PROCNAME("arrayWrite");
 
     if (!filename)
-	return ERROR_INT("filename not defined", procName, 1);
+        return ERROR_INT("filename not defined", procName, 1);
     if (!operation)
-	return ERROR_INT("operation not defined", procName, 1);
+        return ERROR_INT("operation not defined", procName, 1);
     if (!data)
-	return ERROR_INT("data not defined", procName, 1);
+        return ERROR_INT("data not defined", procName, 1);
     if (nbytes <= 0)
-	return ERROR_INT("nbytes must be > 0", procName, 1);
+        return ERROR_INT("nbytes must be > 0", procName, 1);
 
     if (!strcmp(operation, "w") && !strcmp(operation, "a"))
-	return ERROR_INT("operation not one of {'w','a'}", procName, 1);
+        return ERROR_INT("operation not one of {'w','a'}", procName, 1);
 
     if ((fp = fopen(filename, operation)) == NULL)
-	return ERROR_INT("stream not opened", procName, 1);
+        return ERROR_INT("stream not opened", procName, 1);
     fwrite(data, 1, nbytes, fp);
     fclose(fp);
 
@@ -997,19 +1096,19 @@ FILE  *fp;
     PROCNAME("fopenReadStream");
 
     if (!filename)
-	return (FILE *)ERROR_PTR("filename not defined", procName, NULL);
+        return (FILE *)ERROR_PTR("filename not defined", procName, NULL);
 
-	/* try input filename */
+        /* Try input filename */
     if ((fp = fopen(filename, "rb")))
-	return fp;
+        return fp;
 
-	/* else, strip directory and try locally */
+        /* Else, strip directory and try locally */
     splitPathAtDirectory(filename, NULL, &tail);
     if ((fp = fopen(tail, "rb"))) {
-	FREE((void *)tail);
-	return fp;
+        FREE(tail);
+        return fp;
     }
-    FREE((void *)tail);
+    FREE(tail);
 
     return (FILE *)ERROR_PTR("file not found", procName, NULL);
 }
@@ -1055,22 +1154,22 @@ char  *cpathname, *lastslash;
 
     cpathname = stringNew(pathname);
     if ((lastslash = strrchr(cpathname, '/'))) {
-	if (ptail)
-	    *ptail = stringNew(lastslash + 1);
-	if (pdir) {
-	    *(lastslash + 1) = '\0';
-	    *pdir = cpathname;
-	}
-	else
-	    FREE((void *)cpathname);
+        if (ptail)
+            *ptail = stringNew(lastslash + 1);
+        if (pdir) {
+            *(lastslash + 1) = '\0';
+            *pdir = cpathname;
+        }
+        else
+            FREE(cpathname);
     }
     else {  /* no directory */
-	if (pdir)
-	    *pdir = stringNew("");
-	if (ptail)
-	    *ptail = cpathname;
-	else
-	    FREE((void *)cpathname);
+        if (pdir)
+            *pdir = stringNew("");
+        if (ptail)
+            *ptail = cpathname;
+        else
+            FREE(cpathname);
     }
 
     return 0;
@@ -1088,16 +1187,16 @@ char  *cpathname, *lastslash;
  *                        there is no extension, it returns the empty string)
  *      Return: 0 if OK, 1 on error
  *       
- *  Note: (1) if you only want the extension, input null for
- *            the basename ptr.
- *        (2) if you only want the basename without extension,
- *            input null for the extension ptr.
- *        (3) This function makes decisions based only on the lexical
- *            structure of the input.  Examples:
- *              /usr/tmp/abc.jpg  -->  basename: /usr/tmp/abc   ext: .jpg
- *              /usr/tmp/.jpg  -->  basename: /usr/tmp/   tail: .jpg
- *              /usr/tmp.jpg/  -->  basename: /usr/tmp.jpg/   tail: [empty str]
- *              ./.jpg  -->  basename: ./   tail: .jpg
+ *  Notes:
+ *      (1) If you only want the extension, input null for the basename ptr.
+ *      (2) If you only want the basename without extension, input null
+ *          for the extension ptr.
+ *      (3) This function makes decisions based only on the lexical
+ *          structure of the input.  Examples:
+ *            /usr/tmp/abc.jpg  -->  basename: /usr/tmp/abc   ext: .jpg
+ *            /usr/tmp/.jpg  -->  basename: /usr/tmp/   tail: .jpg
+ *            /usr/tmp.jpg/  -->  basename: /usr/tmp.jpg/   tail: [empty str]
+ *            ./.jpg  -->  basename: ./   tail: .jpg
  */
 l_int32
 splitPathAtExtension(const char  *pathname,
@@ -1133,8 +1232,8 @@ char   empty[4] = "";
         if (pbasename)
             *pbasename = stringNew(pathname);
     }
-    FREE((void *)dir);
-    FREE((void *)tail);
+    FREE(dir);
+    FREE(tail);
     return 0;
 }
 
@@ -1156,20 +1255,20 @@ char    *charbuf;
     PROCNAME("genPathname");
 
     if (!dir)
-	return (char *)ERROR_PTR("dir not defined", procName, NULL);
+        return (char *)ERROR_PTR("dir not defined", procName, NULL);
     if (!fname)
-	return (char *)ERROR_PTR("fname not defined", procName, NULL);
+        return (char *)ERROR_PTR("fname not defined", procName, NULL);
 
     dirlen = strlen(dir);
     namelen = strlen(fname);
     if ((charbuf = (char *)CALLOC(dirlen + namelen + 10, sizeof(char)))
             == NULL)
-	return (char *)ERROR_PTR("charbuf not made", procName, NULL);
+        return (char *)ERROR_PTR("charbuf not made", procName, NULL);
 
     if (dir[dirlen - 1] != '/')
-	sprintf(charbuf, "%s/", dir);
+        sprintf(charbuf, "%s/", dir);
     else
-	strcpy(charbuf, dir);
+        strcpy(charbuf, dir);
     strcat(charbuf, fname);
     return charbuf;
 }
@@ -1185,7 +1284,7 @@ char    *charbuf;
  *      ....
  *      fprintf(stderr, "Elapsed time = %7.3f sec\n", stopTimer());
  */
-#ifndef __MINGW32__ 
+#if !defined(__MINGW32__) && !defined(_WIN32)
 
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -1247,8 +1346,8 @@ FILETIME  start, stop, kernel, user;
     utime_after.LowPart  = user.dwLowDateTime;
     utime_after.HighPart = user.dwHighDateTime;
 
-    return ((l_float32)(utime_after.QuadPart - utime_before.QuadPart) /
-	     10000000.0);
+    return ((l_float32)(signed)(utime_after.QuadPart - utime_before.QuadPart) /
+             10000000.0);
 }
 
 #endif
